#include "downloader.h"
#include "aria_error.h"

Downloader::Downloader(int id , QObject *parent) :
    QObject(parent)
{
    proc = new QProcess (this);
    bar = new QProgressBar ();
    eta = new QLabel ();
    spd = new QLabel ();
    size = new QLabel ();
    name = new QLabel ();

    this->id = id;

    connect (proc , SIGNAL(readyReadStandardOutput()) , SLOT(readyRead()));
    connect (proc , SIGNAL(readyReadStandardError()) , SLOT(errorReady()));
}

void Downloader::errorReady()
{
    qDebug() << "ERR: " << proc->readAllStandardError();
}

void Downloader::readyRead()
{
    const QStringList & output = QString (proc->readAllStandardOutput()).split('\n');

    foreach (QString line , output)
    {
        if ( line.startsWith("Exception:") )
        {
            static const QRegExp errorCodeNum ("errorCode=([0-9]+)");
            if ( errorCodeNum.indexIn(line) != -1 )
            {
                int idx = errorCodeNum.cap(1).toInt();
                int tot = sizeof ( aria_error ) / sizeof ( char* );

                if ( idx == 0 )
                {
                    emit finished (id);
                    return;
                }

                if ( idx < tot  && idx > 0 )
                {
                    emit error ( id , aria_error[idx - 1] );
                }
                else
                    emit error ( id , QString ("Unknown error: %1").arg(idx) );
            }
            else
                emit error ( id , line );

            return;
        }

        if ( line.startsWith("FILE:") )
        {
            name->setText(SPACES + line.remove(QRegExp(".*/")) + SPACES);
        }

        if ( line.indexOf("download completed") != -1 )
        {

            spd->setText( SPACES + QString("0 Kib/s") + SPACES);
            bar->setValue(100);
            eta->setText( SPACES + QString("00:00") + SPACES);

            emit finished(id);

            return;
        }

        static const QRegExp Progress ("SIZE:([0-9.,]+[A-Za-z]+)/([0-9.,]+[A-Za-z]+)\\(([0-9]+)%\\).*SPD:([0-9.]+[A-Za-z]+).*ETA:(.*)\\]");
        if ( Progress.indexIn(line) != -1 )
        {
            spd->setText( SPACES + Progress.cap(4) + SPACES);
            size->setText( SPACES + Progress.cap(1) + " / " + Progress.cap(2) + SPACES);
            bar->setValue(Progress.cap(3).toInt());
            eta->setText( SPACES + Progress.cap(5) + SPACES);

            emit infoUpdated();
        }
    }
}

void Downloader::setFolder(const QString &folder)
{
    cachedFolder = folder;
}

void Downloader::begin(const QString &url)
{
    cachedUrl = url;
    start();
}

void Downloader::start()
{
    QStringList arguments;
    arguments << "-d" << cachedFolder << "-s10" << "-j10" << "-c" << cachedUrl
                 << "--summary-interval=1";

    arguments.append(aria2cOpt.split(" "));

    proc->start("aria2c" , arguments);
}

void Downloader::stop()
{
    proc->terminate();
}
